package de.mutschler.rpdl;

import de.mutschler.util.LogObj;
import java.util.*;
import java.text.*;

/**
 * This class represents an entry in the historylist. It can be compared againsts its date.
 */
public class RpdlHistoryEntry extends LogObj implements Comparable,java.io.Serializable
{
	static final long serialVersionUID = 5208662522816760002L;
	//----------------------------------------------------------------------
	//------- Attributes
	//----------------------------------------------------------------------
	
	/** Store the date for this entry. */
	private Date date;
	
	/** This is the info-text for the date. */
	private String text;
	
	/** The short description, or title */
	private String title;
	
	//----------------------------------------------------------------------
	//------- Constructor(s)
	//----------------------------------------------------------------------
	
	/**
	 * Create a new HistoryEntry.
	 * @param date the date for the entry
	 * @param text the text
	 */
	public RpdlHistoryEntry(Date date, String title, String text) {
		this.date = date;
		this.title = title;
		this.text = text;
	}

	//----------------------------------------------------------------------
	//------- Accessors
	//----------------------------------------------------------------------
	
	/** Retrieve the date for this entry. */
	public Date getDate() {return date;}

	/** Retrieve the title for this entry. */
	public String getTitle() {return title;}

	/**
	 * Get the formatted date. The format is according to
	 * the settings in RpdlConfig.
	 * @see RpdlConfig#getDateFormat
	 */
	public String getDateFormatted() {
		return RpdlBase.getConfig().getDateFormat().format(date);
	}

	/** Retrieve the text for this entry. */
	public String getText() {return text;}
	
	//----------------------------------------------------------------------
	//------- Overridden system methods
	//----------------------------------------------------------------------
	
	/** Compare the date of the entries */
	public int compareTo(Object o) {
		return date.compareTo(((RpdlHistoryEntry)o).getDate());
	}
	
	/** Compare the date of the entries */
	public boolean equals(Object o) {
		return date.equals(((RpdlHistoryEntry)o).getDate());
	}
	
	public String toString() {
		return "[" + getDateFormatted() + "] \"" + getText() + "\"";
	}
}