package de.mutschler.rpdl.basicvalue;

import de.mutschler.util.LogObj;

public class ValueDice extends BasicValue implements Cloneable,java.io.Serializable
{
	static final long serialVersionUID = -3427941901789374693L;
	//----------------------------------------------------------------------
	//------- Attributes
	//----------------------------------------------------------------------

	/** This array defines the values of the dices, and all
	 * possible values */
	private static int[] dicevalue = {1,4,6,8,10,12,20,30,100};
	
	/** This is the letter which specifies the dice in e.g. 4D6. */
	private final String diceSign = "W";

	/** internat array to store the dices */
	private int[] dices;

	//----------------------------------------------------------------------
	//------- Constructor(s)
	//----------------------------------------------------------------------

	private final static int NUMDICES = 9;
	/**
	 * This is the main constructor. It initializes all to zero.
	 */
	public ValueDice() {
		super();
		dices = new int[NUMDICES];		// can't use dicevalue.length here!!!!!!!!
		for (int i=0; i<NUMDICES; i++) {
			dices[i] = 0;
		}
	}

	/** construct from a string */
	public ValueDice(String s)
	{
		this();
		addDice(s);
	}

	public void setValue(BasicValue t) throws RpdlConversionException
	{
		if (TYPENAME.equals(t.getType())) {
			for (int i=0; i<NUMDICES; i++) {
				dices[i] = 0;
			}
			add((ValueDice)t);
			return;
		}
		// the superclass can throw the excpeption
		super.setValue(t);
	}
	//----------------------------------------------------------------------
	//------- mathematical operations
	//----------------------------------------------------------------------

	/** Perform a calculation. */
	public BasicValue calculate(int op, BasicValue t) throws RpdlConversionException
	{
		String type = ((t==null)?null:t.getType());
		ValueDice d;
		switch(op) {
			case TO_DICE:
				return this;
			case OP_NEG:								// !DICE -> DICE
				d = new ValueDice();
				d.sub(this);
				return d;
			case OP_ADD:
				if (ValueInt.TYPENAME.equals(type)) {	// DICE + INT -> DICE
					d=(ValueDice)clone();
					d.addDice(1, ((ValueInt)t).getValue());
					return d;
				}
				if (ValueString.TYPENAME.equals(type)) {	// DICE + STRING -> STRING
					return new ValueString(getStringValue() + ((ValueString)t).getValue());
				}
				if (ValueDice.TYPENAME.equals(type)) {	// DICE + DICE -> DICE
					d=(ValueDice)clone();
					d.add((ValueDice)t);
					return d;
				}
				break;
			case OP_SUB:
				if (ValueInt.TYPENAME.equals(type)) {	// DICE - INT -> DICE
					d=(ValueDice)clone();
					d.addDice(1, -((ValueInt)t).getValue());
					return d;
				}
				if (ValueDice.TYPENAME.equals(type)) {	// DICE - DICE -> DICE
					d=(ValueDice)clone();
					d.sub((ValueDice)t);
					return d;
				}
				break;
			case OP_MUL:
				if (ValueInt.TYPENAME.equals(type)) {	// DICE * INT -> DICE
					d=(ValueDice)clone();
					d.multiply(((ValueInt)t).getValue());
					return d;
				}
				break;
			case COMP_EQ:								// DICE == DICE -> BOOL
				if (ValueDice.TYPENAME.equals(type)) {
					return new ValueBool(equals(t));
				}
				break;
		}
		return super.calculate(op, t);	// let the superclass throw the exception
	}
	
	/**
	 * For the static type-checking. Return the expected type for an
	 * operation.
	 */
	public String getResultType(int op, String type) {
		switch(op) {
			case TO_DICE:	return ValueDice.TYPENAME;
			case OP_NEG:	return ValueDice.TYPENAME;	// !DICE -> DICE
			case OP_ADD:
				if (ValueInt.TYPENAME.equals(type)) {	// DICE + INT -> DICE
					return ValueDice.TYPENAME;
				}
				if (ValueString.TYPENAME.equals(type)) {	// DICE + STRING -> STRING
					return ValueString.TYPENAME;
				}
				if (ValueDice.TYPENAME.equals(type)) {	// DICE + DICE -> DICE
					return ValueDice.TYPENAME;
				}
				break;
			case OP_SUB:
				if (ValueInt.TYPENAME.equals(type)) {	// DICE - INT -> DICE
					return ValueDice.TYPENAME;
				}
				if (ValueDice.TYPENAME.equals(type)) {	// DICE - DICE -> DICE
					return ValueDice.TYPENAME;
				}
				break;
			case OP_MUL:
				if (ValueInt.TYPENAME.equals(type)) {	// DICE * INT -> DICE
					return ValueDice.TYPENAME;
				}
				break;
			case COMP_EQ:								// DICE == DICE -> BOOL
				if (ValueDice.TYPENAME.equals(type)) {
					return ValueBool.TYPENAME;
				}
				break;
		}
		return super.getResultType(op, type);	// let the superclass return the error
	}
	//----------------------------------------------------------------------
	//------- Accessors
	//----------------------------------------------------------------------
	
	/** The name of this type. */
	public static final String TYPENAME="DICE";
	/** The name of this type. */
	public String getType() {return TYPENAME;}

	/** Return the cast operation for casting into the same value type. */
	public int getCastOp() {return BasicValue.TO_DICE;}

	/** Get a string-representation of the value. This is used for concatenation. */
	public String getStringValue() {return toString();}

	//----------------------------------------------------------------------
	//------- Methods
	//----------------------------------------------------------------------
	
	
	public void addDice(String s) {
		// parse the string
		int state = 0;
		int curval = 0;
		int curdice = 0;
		boolean addVal = true;
		for (int i=0; i<s.length(); i++) {
			char c = s.charAt(i);
//			debug("scanning " + c + " @" + i);
			if (c == ' ') continue;
			switch (state) {
				case 0:	// starting
					// if we have a D or we, the continue with specifying the dice
					if (c == 'D' || c == 'W' || c == 'd' || c == 'w')  {
						state = 1;
						break;
					}
					if (c == '+') {
						addDice (1, addVal?curval:-curval);
						curdice = 0; curval = 0;
						addVal = true;
						state = 0;
						break;
					}
					if (c == '-') {
						addDice (1, addVal?curval:-curval);
						curdice = 0; curval = 0;
						addVal = false;
						state = 0;
						break;
					}
					// count the value
					if (!Character.isDigit(c))
						throw new NumberFormatException("Invalid dice '" + s + "' at pos " + i);
					curval = 10*curval + c-'0';
					break;
				case 1:
					if (c == '+') {
						addDice (curdice, addVal?curval:-curval);
						curdice = 0; curval = 0;
						addVal = true;
						state = 0;
						break;
					}
					if (c == '-') {
						addDice (curdice, addVal?curval:-curval);
						curdice = 0; curval = 0;
						addVal = false;
						state = 0;
						break;
					}
					// count the value
					if (!Character.isDigit(c))
						throw new NumberFormatException("Invalid dice '" + s + "' at pos " + i);
					curdice = 10*curdice + c-'0';
					break;
			}
		}
		addDice (curdice==0?1:curdice, addVal?curval:-curval);
	}	
	
	/**
	 * Add a number of dices.
	 * @param dice A dicevalue of 1,4,6,8,10,12,20,30,100
	 * @param count the number of this type of dices
	 */
	public void addDice(int dice, int count) {
		debug("Adding dice " + count + "W" + dice);
		dices[getOffset(dice)] += count;
	}
	
	/** Subtract another dice to this one
	 * @return the current dice */
	public ValueDice sub(ValueDice d) {
		for (int i=0; i<dicevalue.length; i++) {
			dices[i] -= d.dices[i];
		}
		return this;
	}
	
	/** Add another dice to this one
	 * @return the current dice */
	public ValueDice add(ValueDice d) {
		for (int i=0; i<dicevalue.length; i++) {
			dices[i] += d.dices[i];
		}
		return this;
	}
	
	/** Multiply all the included dices
	 * @return the current dice */
	public ValueDice multiply(int mul) {
		for (int i=0; i<dicevalue.length; i++) {
			dices[i] *= mul;
		}
		return this;
	}
	
	/**
	 * Add an integer. This integer is treated as a simple value.
	 * This is the same as addDice(1, i);
	 * @return the current dice 
	 */
	public ValueDice add(int i) {
		addDice(1, i);
		return this;
	}
	
	private int getOffset(int dice) {
		for (int i=0; i<dicevalue.length; i++) {
			if (dicevalue[i] == dice)
				return i; 
		}
		throw new NumberFormatException("Invalid dice '" + dice +"'");
	}
	
	//----------------------------------------------------------------------
	//------- Overridden system methods
	//----------------------------------------------------------------------
	
	/** Compare two dices. this compares the number of all possible dices.*/
	public boolean equals(Object o) {
		if (o instanceof ValueDice) {
			ValueDice d = (ValueDice)o;
			for (int i=0; i<dicevalue.length; i++) {
				if (dices[i] != d.dices[i])
					return false;
			}
			return true;
		}
		return false;
	}
	
	public String toString() {
		StringBuffer sb = new StringBuffer(10);
		boolean first = true;
		for (int x=1; x<=dicevalue.length; x++) {
			int i = (x==dicevalue.length)?0:x;
			if (dices[i] != 0) {
				char dicesign = dices[i]<0?'-':'+';
				if (first) {
					if (dicesign=='-')
						sb.append(dicesign).append(' ');
				}
				else {
					sb.append(' ').append(dicesign).append(' ');
				}
				sb.append(dicesign=='+'?dices[i]:-dices[i]);
				if (dicevalue[i] != 1) {
					sb.append(diceSign);
					sb.append(dicevalue[i]);
				}
				first = false;
			}
		}
		return sb.toString();
	}

	public Object clone() {
		ValueDice d = new ValueDice();
		d.add(this);
		return d;
	}

	//----------------------------------------------------------------------
	//------- testing
	//----------------------------------------------------------------------
	
	 static public void main(String[] args) {
		new ValueDice().test();
	}
	
	private void test() {
		ValueDice d;
		debug("Testing dices");
		debug ("Dice '2' = " + new ValueDice("2"));
		String td = "2W4 + 5W8 + 5";
		debug("creating dice from " + td);
		d = new ValueDice(td);
		debug("result is " + d);
		d.add(new ValueDice("3W6"));
		debug("added 3W6: " + d);
	}
}