package de.mutschler.rpdl.basicvalue;

import de.mutschler.util.LString;

public class ValueString extends BasicValue implements java.io.Serializable
{
	static final long serialVersionUID = -5995183277432545686L;
	//----------------------------------------------------------------------
	//------- Attributes
	//----------------------------------------------------------------------

	/** Here the actual value is stored */
	private LString value;
	
	//----------------------------------------------------------------------
	//------- Constructor(s)
	//----------------------------------------------------------------------
	public ValueString() {this("");}
	
	public ValueString (String s) {
		super(s);
		value = new LString(s);
	}
	
	public ValueString (LString ls) {
		super(ls.getLocaleString());
		value = new LString(ls);
	}
	
	//----------------------------------------------------------------------
	//------- Methods
	//----------------------------------------------------------------------
	
	/** The name of this type. */
	public static final String TYPENAME="STRING";
	/** The name of this type. */
	public String getType() {return TYPENAME;}
	
	/** Return the cast operation for casting into the same value type. */
	public int getCastOp() {return BasicValue.TO_STRING;}

	/** Get the actual value of this type. */
	public final String getValue() {return value.getLocaleString();}

	/** Get a string-representation of the value. This is used for concatenation. */
	public String getStringValue() {return getValue();}
	
	public void setValue(BasicValue t) throws RpdlConversionException {
		String s = t.getStringValue();
		if (s == null) {
			error("basicvalue " + t + " return null-string. Type= + " + t.getType());
		}
		value = new LString(s);
	}
	
	//----------------------------------------------------------------------
	//------- mathematical operations
	//----------------------------------------------------------------------

	/** Perform a calculation. */
	public BasicValue calculate(int op, BasicValue t) throws RpdlConversionException
	{
		String type = ((t==null)?null:t.getType());
		switch(op) {
			case TO_STRING:
				return this;
			case OP_ADD:									// STRING + everything -> STRING
				return new ValueString(getValue() + t.getStringValue());
			case COMP_EQ:
				if (ValueString.TYPENAME.equals(type)) {	// STRING = STRING -> BOOL
					return new ValueBool(getValue().equalsIgnoreCase(((ValueString)t).getValue()));
				}
				break;
			case COMP_NE:
				if (ValueString.TYPENAME.equals(type)) {	// STRING <> STRING -> BOOL
					return new ValueBool(!getValue().equalsIgnoreCase(((ValueString)t).getValue()));
				}
				break;
			case COMP_LE:
				if (ValueString.TYPENAME.equals(type)) {	// STRING <= STRING -> BOOL
					return new ValueBool(getValue().compareToIgnoreCase(((ValueString)t).getValue()) <= 0);
				}
				break;
			case COMP_LT:
				if (ValueString.TYPENAME.equals(type)) {	// STRING < STRING -> BOOL
					return new ValueBool(getValue().compareToIgnoreCase(((ValueString)t).getValue()) < 0);
				}
				break;
			case COMP_GE:
				if (ValueString.TYPENAME.equals(type)) {	// STRING >= STRING -> BOOL
					return new ValueBool(getValue().compareToIgnoreCase(((ValueString)t).getValue()) >= 0);
				}
				break;
			case COMP_GT:
				if (ValueString.TYPENAME.equals(type)) {	// STRING > STRING -> BOOL
					return new ValueBool(getValue().compareToIgnoreCase(((ValueString)t).getValue()) > 0);
				}
				break;
		}
		return super.calculate(op, t);	// let the superclass throw the exception
	}

	/**
	 * For the static type-checking. Return the expected type for an
	 * operation.
	 */
	public String getResultType(int op, String type) {
		switch(op) {
			case TO_STRING:
				return ValueString.TYPENAME;
			case OP_ADD:								// STRING + everything -> STRING
				return ValueString.TYPENAME;
			case COMP_EQ:
				if (ValueString.TYPENAME.equals(type)) {	// STRING = STRING -> BOOL
					return ValueBool.TYPENAME;
				}
				break;
			case COMP_NE:
				if (ValueString.TYPENAME.equals(type)) {	// STRING <> STRING -> BOOL
					return ValueBool.TYPENAME;
				}
				break;
			case COMP_LE:
				if (ValueString.TYPENAME.equals(type)) {	// STRING <= STRING -> BOOL
					return ValueBool.TYPENAME;
				}
				break;
			case COMP_LT:
				if (ValueString.TYPENAME.equals(type)) {	// STRING < STRING -> BOOL
					return ValueBool.TYPENAME;
				}
				break;
			case COMP_GE:
				if (ValueString.TYPENAME.equals(type)) {	// STRING >= STRING -> BOOL
					return ValueBool.TYPENAME;
				}
				break;
			case COMP_GT:
				if (ValueString.TYPENAME.equals(type)) {	// STRING > STRING -> BOOL
					return ValueBool.TYPENAME;
				}
				break;
		}
		return super.getResultType(op, type);	// let the superclass return the error
	}

	//----------------------------------------------------------------------
	//------- overridden methods
	//----------------------------------------------------------------------
	/** Create a copy of this object */
	public Object clone() {return new ValueString(value);}
		
}
