package de.mutschler.rpdl.statement;

import de.mutschler.rpdl.*;
import de.mutschler.rpdl.entity.*;

/**
 * Statement to add a new type to the object.
 */
public class Stmt_AddType extends Stmt implements java.io.Serializable
{
	static final long serialVersionUID = 1868626635012490030L;
	//----------------------------------------------------------------------
	//------- Attributes
	//----------------------------------------------------------------------

	/** The name of the new type. */
	private String typeName;
	
	//----------------------------------------------------------------------
	//------- Constructor(s)
	//----------------------------------------------------------------------

	/**
	 * @param name the name of the new type
	 */
	public Stmt_AddType(String name) {
		super();
		this.typeName = name;
	}	

	//----------------------------------------------------------------------
	//------- Overridden methods of the superclass
	//----------------------------------------------------------------------

	/**
	 * This method can be overwritten, and then used to check the validity
	 * of the statement. This method is called from the addCommand() of CmdCrType & CmdCrObj
	 * The default implementaion does nothing.
	 */
	public void checkStatement(RpdlBase main, Entity e) throws RpdlException {
		super.checkStatement(main, e);
		RpdlType type = main.getType(typeName);
		if (type == null)
			throw new RpdlException("Type '" + typeName + "' doesn't exist!", getOrigin());
		// now, add the type to the specified object
		if (!(e instanceof RpdlObject))
			throw new RpdlException("Error: entity " + e.getName() + " is no RpdlObject!", getOrigin());
		RpdlObject obj = (RpdlObject)e;
		obj.addType(type);
	}
	
	public String toString() {
		return "ADD TYPE " + typeName;
	}
}