package de.mutschler.rpdl.statement;

import de.mutschler.rpdl.*;
import de.mutschler.rpdl.entity.*;

public class Stmt_Assert extends Stmt_BoolFormula implements java.io.Serializable
{
	static final long serialVersionUID = -5116874584574633038L;
	//----------------------------------------------------------------------
	//------- Attributes
	//----------------------------------------------------------------------
	
	/** Assertion of type error. */
	public static final int ERROR=0;
	/** Assertion of type warning. */
	public static final int WARN=1;
	
	/** type of assertion. */
	private int type;
	
	/** The error-message to display, if the formula returns false. */
	private RpdlFormula errorMessage;
	
	//----------------------------------------------------------------------
	//------- Constructor(s)
	//----------------------------------------------------------------------


	/**
	 * Create a new assert statement.
	 * @param formula the formula to calculate
	 */

	public Stmt_Assert(RpdlFormula formula, int type, RpdlFormula msg) {
		super(formula);
		this.type = type;
		errorMessage = msg;
	}

	//----------------------------------------------------------------------
	//------- Accessors
	//----------------------------------------------------------------------
	/** Get the assertion type. */
	public int getAssertType() {return type;}
	
	/** Get the message formula. */
	public RpdlFormula getMessageFormula() {return errorMessage;}
	
	
	//----------------------------------------------------------------------
	//------- Methods
	//----------------------------------------------------------------------
	
	public static String getAssertTypeName(int type) {
		switch(type) {
			case ERROR:	return "ERROR";
			case WARN:	return "WARN";
			default: return "UNKNOWN";
		}
	}
			
	/** Get the execution priority. */
	public int getPriority() {return PRI_LOW;}

	/**
	 * Excecute the statement. Here the actual calculation for statements
	 * are done. 
	 */
	public void execute(RpdlBase main, RpdlObject obj) throws RpdlException {
		super.execute(main, obj);
		// now evaluate the formula:
		if (!calcFormula(main, obj)) {
			throw new RpdlAssertionException("ASSERT " + getAssertTypeName(getAssertType()) + ": " +
				getMessageFormula().calc(main, obj).getStringValue());
		}
	}

	//----------------------------------------------------------------------
	//------- Overridden system methods
	//----------------------------------------------------------------------
	
	public String toString() {
		return	"ASSERT " + getAssertTypeName(getAssertType()) +
				" (" + getFormula() + ") " +
				getMessageFormula() + ";";
	}
}