package de.mutschler.rpdl.statement;

import de.mutschler.rpdl.*;
import de.mutschler.rpdl.entity.*;
import de.mutschler.rpdl.basicvalue.*;
import de.mutschler.rpdl.calc.*;

/**
 * This statement works with a formula, which results in a boolean value.
 */
public abstract class Stmt_BoolFormula extends Stmt_Formula implements java.io.Serializable
{
	static final long serialVersionUID = -8461178540455971477L;
	//----------------------------------------------------------------------
	//------- Constructor(s)
	//----------------------------------------------------------------------

	/**
	 * Create a new statement. This checks, of the result
	 * is of boolean type.
	 * @param formula the formula to calculate
	 */

	public Stmt_BoolFormula(RpdlFormula formula) {
		super(formula);
	}

	//----------------------------------------------------------------------
	//------- Accessors
	//----------------------------------------------------------------------
	
	//----------------------------------------------------------------------
	//------- Overridden methods of the superclass
	//----------------------------------------------------------------------
	/**
	 * This method can be overwritten, and then used to check the validity
	 * of the statement. This method is called from the addCommand() of CmdCrType & CmdCrObj
	 * The default implementaion does nothing.
	 */
	public void checkStatement(RpdlBase base, Entity e) throws RpdlException {
		super.checkStatement(base, e);
		// add a type-cast to the formula, if needed
		RpdlFormula f = getFormula();
		String fmlType = f.getResultType(base, e);
		if (!fmlType.equals(ValueBool.TYPENAME)) {
			debug("adding cast to formula: " + f);
			f = new FormulaMath(BasicValue.getCastOperation(ValueBool.TYPENAME), f, null);
			debug("cast to formula: " + f);
			setFormula(f);
		}

		// check the formula now again with the cast
		String type = f.getResultType(base, e);
		if (!ValueBool.TYPENAME.equals(type)) {
			throw new RpdlException("Formula doesn't return a boolean value!" + 
									" Statement: '" + toString() + "'", getOrigin());
		}
	}
	
	/** Method to execute the formula and get the result.
	 * You can use this method in your execute() method.
	 */
	public boolean calcFormula(RpdlBase main, RpdlObject obj) throws RpdlException {
		BasicValue res = getFormula().calc(main, obj);
		debug("result of bool formula '" + getFormula() + "' is " + res);
		if (res instanceof ValueBool) {
			return ((ValueBool)res).getValue();
		}
		throw new RpdlException("Formula '" + getFormula() + "' didn't return a boolean value!", getOrigin());
	}

	/**
	 * This Method is called, when a statement is triggered. You can add
	 * things to the CalcHandler in this method.
	 *
	 * For the assertion, a CalcStmt will be executed.
	 */
	public void trigger(RpdlBase main, StmtHandle sh) throws RpdlException {
		main.addCalcItem(new CalcStmt(sh));
	}
}