package de.mutschler.rpdl.statement;

import de.mutschler.rpdl.entity.*;
import de.mutschler.rpdl.*;
import de.mutschler.rpdl.basicvalue.*;
import de.mutschler.util.Log;
import de.mutschler.rpdl.calc.*;

public class Stmt_Require extends Stmt_BoolFormula implements java.io.Serializable
{
	static final long serialVersionUID = 6091320945967474779L;
	//----------------------------------------------------------------------
	//------- Attributes
	//----------------------------------------------------------------------
	
	/** The error-message to display, if the formula returns false. */
	private RpdlFormula errorMessage;
	
	//----------------------------------------------------------------------
	//------- Constructor(s)
	//----------------------------------------------------------------------

	/**
	 * Create a new require statement.
	 * @param formula the formula to calculate
	 */

	public Stmt_Require(RpdlFormula formula, RpdlFormula msg) {
		super(formula);
		errorMessage = msg;
	}

	//----------------------------------------------------------------------
	//------- Accessors
	//----------------------------------------------------------------------
	
	/** Get the message formula. */
	public RpdlFormula getMessageFormula() {return errorMessage;}

	/** Get the execution priority. */
	public int getPriority() {return PRI_HIGH;}	// REQUIRE should be the first command executed

	/**
	 * This Method checks the result of a formula
	 */
	public void checkStatement(RpdlBase base, Entity e) throws RpdlException {
		super.checkStatement(base, e);
		try {
			String type = getMessageFormula().getResultType(base, e);
			if (type.equals(BasicValue.TYPE_ERR))
				throw new RpdlException("Formula is incorrect!" +
										" Statement: '" + toString() + "'", getOrigin());
		} catch (Exception ex) {
			Log.exception(ex);
			throw new RpdlException("Error in formula " + toString() + ": "+ ex.getMessage(), getOrigin());
		}
	}

	/**
	 * Excecute the statement. Here the actual calculation for statements
	 * are done. 
	 */
	public void execute(RpdlBase main, RpdlObject obj) throws RpdlException {
		super.execute(main, obj);
		// calculate the formula
		BasicValue res = getFormula().calc(main, obj);
		if (!((ValueBool)res).getValue()) {
			res = getMessageFormula().calc(main, obj);
			throw new RpdlAssertionException(res.getStringValue());
		}
	}

	/**
	 * This method is called, when an object is linked/unlinked to the base object.
	 * checks if the formula calculates to true. If not, a RpdlAssertionException is thrown.
	 */
	public void onBaseLink(RpdlBase main, RpdlObject obj, StmtHandle sh, boolean link) throws RpdlException {
		if (link) {
			main.addCalcItem(new CalcStmt(sh));	// execute once
		}
		else {
			// unlink: nothing to do
		}
		// calculate the formula
/*		BasicValue res = getFormula().calc(main, obj);
		if (!((ValueBool)res).getValue()) {
			res = getMessageFormula().calc(main, obj);
			throw new RpdlAssertionException(res.getStringValue());
		}
*/	}
	
	//----------------------------------------------------------------------
	//------- Overridden system methods
	//----------------------------------------------------------------------
	
	public String toString() {
		return "REQUIRE (" + getFormula() + ") " + getMessageFormula() + ";";
	}
}