package de.mutschler.util;

import java.util.Iterator;

/**
 * This class iterates over multiple single iterators.
 * e.g. to iterate over two lists, you can simply use the following code:
 * Iterator iterator() {
 *   Iterator its[] = {list1.iterator(), list2.iterator()};
 *   return new JoinIterator(its);
 * }
 */
public class JoinIterator implements Iterator
{
	private Iterator iterators[];
	private int current;
	
	
	public JoinIterator(Iterator[] its) {
		iterators = its;
		current = 0;
	}

	public boolean hasNext() {
		if (current >= iterators.length) return false;	// no more iterators available
		// check if we have a valid iterator,
		// and if the current Iterator has no next, advance to the next list
		while (iterators[current]==null || !iterators[current].hasNext()) {
			current++;
			if (current >= iterators.length) return false;	// no more iterators available
		}
		return true;
	}
	public Object next() {
		return iterators[current].next();
	}
	public void remove() {
		iterators[current].remove();
	}
}