
/**
 * @author Michael Mutschler
 * @version 1.0
 */
package de.mutschler.util;

import java.io.*;
import java.text.SimpleDateFormat;
import java.util.Date;

/**
 * This class handles all the debug stuff
 */
 
public class Log {
	/** Log type CRITICAL */
	public static final int CRITICAL=0;
	/** Log type ERROR */
	public static final int ERROR=10;
	/** Log type WARN */
	public static final int WARN=20;
	/** Log type DEBUG */
	public static final int DEBUG=30;
	/** Log type DDEBUG. Much debug. */
	public static final int DDEBUG=40;

	/** Current output level. defaults to WARN. */
	private static int logLevel = DEBUG;
	/** Set the logging level for output.
	 * @return the previous log level.
	 */
	public static int setLogLevel(int l) {
		int old = logLevel;
		logLevel = l;
		return old;
	}
	/** Get the current logging level. */
	public static int getLogLevel() {return logLevel;}

	/** Where to output everything */
	private static PrintWriter logOut = new PrintWriter(System.out, true);
	private static PrintWriter logOut2 = null;
	
	
	public static void setLogFile(String file) {
		try {
			logOut2 = new PrintWriter(new FileWriter(file));
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	private static SimpleDateFormat fmt = new SimpleDateFormat("[yyMMdd HH:mm:ss]");
	private static StringBuffer line = new StringBuffer(200);	// static stringbuffer for reusing it
	/** log a string */
	public static synchronized void log(int type, String clsName, String s) {
		if (type <= logLevel) {
			line.setLength(0);
			line.append(getTypeString(type));
	    	// print the standard line-prefix:
//			logOut.print(fmt.format(new Date()));
     		String cn = clsName.substring(clsName.lastIndexOf('.')+1);
			line.append(" (").append(cn).append(") ");
			line.append(s);

			if (type < DEBUG) if (logOut  != null) {logOut. println(line.toString()); logOut.flush();}
			if (logOut2 != null) {logOut2.println(line.toString()); logOut2.flush();}
			
		}
	}

	/** Log an exception. */
	public static void exception(Throwable e) {
		if (logOut  != null) {e.printStackTrace(logOut);  logOut.flush();}
		if (logOut2 != null) {e.printStackTrace(logOut2); logOut2.flush();}
	}

	/** log a string and its stacktrace from the exception */
	public static void log(int type, String clsName, String s, Throwable e) {
		if (type <= logLevel) {
			log(type, clsName, s);
		}
		exception(e);
	}
	
	public static final String CRITICAL_STR="CRIT";
	public static final String ERROR_STR=	"ERR ";
	public static final String WARN_STR=	"WARN";
	public static final String DEBUG_STR=	"DBG ";
	public static final String DDEBUG_STR=	"DDBG";
	public static final String UNKNOWN_STR=	"UNKN";
	
	/** Get a string of the messagetype. This is 4 chars! */
	public static String getTypeString(int type) {
		switch(type) {
			case CRITICAL:	return CRITICAL_STR;
			case ERROR:		return ERROR_STR;
			case WARN:		return WARN_STR;
			case DEBUG:		return DEBUG_STR;
			case DDEBUG:	return DDEBUG_STR;
			default:		return UNKNOWN_STR;
		}
	}
	
	/** Log a string as debug. */
	public static final void debug(String clsName, String s) {log(DEBUG, clsName, s);}
	/** Log a string as error. */
	public static final void error(String clsName, String s) {log(ERROR, clsName, s);}
	/** Log a string as warning. */
	public static final void warn(String clsName, String s) {log(WARN, clsName, s);}

	/** Log a string & exception as debug. */
	public static final void debug(String clsName, String s, Throwable t) {log(DEBUG, clsName, s, t);}
	/** Log a string & exception as error. */
	public static final void error(String clsName, String s, Throwable t) {log(ERROR, clsName, s, t);}
	/** Log a string & exception as warning. */
	public static final void warn(String clsName, String s, Throwable t) {log(WARN, clsName, s, t);}

}